@@ -5,7 +5,7 @@ function(x)
     set_union(set(set()),
               as.set(unlist(lapply(seq_along(x),
                                    function(i)
-                                   apply(combn(as.list(x), i), 2L, as.set)
+                                   apply(combn(.as.list(x), i), 2L, as.set)
                                    ),
                             recursive = FALSE)
                      )
@@ -25,3 +25,16 @@ function(x)
                          )
 }
 
+cset_power <-
+function(x)
+{
+    if (length(x) == 0L) return(set(set()))
+    .make_gset_from_list(c(list(gset()),
+                           unlist(lapply(seq_along(x),
+                                         function(i) cset_combn(x, i)
+                                         ),
+                                  recursive = FALSE)
+                           )
+                         )
+}
+
